def run_function(code_string, in_stream, out_stream):
    # Compile the code string
    compiled_code = compile(code_string, '<string>', 'exec')

    # Create a dictionary to hold the function namespace
    function_namespace = {}

    # Execute the compiled code in the function namespace
    exec(compiled_code, function_namespace)

    # Get the function from the namespace
    function = function_namespace.get('file_convert')

    # Call the function with the given parameter
    result = function(in_stream, out_stream)

    # Return the result
    return result






import sys
from dotenv import load_dotenv
from charmr import ai
import argparse
from charmr.load_code import run_function
from charmr import alias_manager


def parse_arguments():
    parser = argparse.ArgumentParser(description='Charmr - Magical File Converter')
    parser.add_argument('--input_file', '-i', type=str, help='Input file', required=False)
    parser.add_argument('--output_file', '-o', type=str, help='Output file', required=False)
    parser.add_argument('--conversion', '-c', type=str, help='Conversion description', required=False)
    parser.add_argument('--alias', '-a', type=str, help='Conversion alias, for reuse', required=False)
    parser.add_argument('--view_code_only', '-v', help='View code without running', action='store_true')
    parser.add_argument('--gpt_4', '-4', help='Use GPT-4', action='store_true')
    parser.add_argument('--include_input_rows', '-r', type=int, help='Include N input rows in prompt', required=False)
    args = parser.parse_args()
    return args


def read_input_file(file_path):
    with open(file_path, 'r') as f:
        file_content = f.read()
    return file_content


def read_first_n_lines(file_path, n):
    with open(file_path, 'r') as f:
        lines = []
        for i in range(n):
            line = f.readline()
            if line:
                lines.append(line)
            else:
                break
    return lines


def send_stream_to_stdout(stream):
    for line in stream:
        sys.stdout.write(line)


def main():
    load_dotenv()
    args = parse_arguments()

    if args.include_input_rows:
        first_lines = read_first_n_lines(args.input_file, args.include_input_rows)
    else:
        first_lines = None

    if args.gpt_4:
        model_name = "gpt-4"
    else:
        model_name = "gpt-3.5-turbo"

    if args.conversion:
        conversion_code = ai.get_code(args.conversion, model=model_name, header_rows=first_lines)

        if args.alias:
            alias_manager.save_code(conversion_code, args.alias)
    else:
        if not args.alias:
            raise Exception("Alias must be provided when no conversion defined")
        else:
            conversion_code = alias_manager.load_code(args.alias)

    if not args.view_code_only:
        if args.input_file:
            in_stream = open(args.input_file, "r")
        else:
            in_stream = sys.stdin

        if args.output_file:
            out_stream = open(args.output_file, "w")
        else:
            out_stream = sys.stdout

        run_function(conversion_code, in_stream, out_stream)
        if args.input_file:
            in_stream.close()
        if args.output_file:
            out_stream.close()

    else:
        print(conversion_code)

    # print(result)


if __name__ == "__main__":
    main()



import openai
import os

SYSTEM = """you are a file converter code generator. 
You will get a description of the source and target files format, and any conversion, manipulation, 
filtering or transformation to apply on the source to generate the target. 
You will output a python function.
The source input is a stream.
The target output is a stream as well.
The function name will be "file_convert". 
function signature is "file_convert(input_stream, output_stream)"
Generate only the code, with no comments. 
Don't generate any wrapper to the code. 
The output should be compilable as python code, version 3.8.
prefer string manipulation over using external packages where possible. 
Do not output any explanation."""


prompt_with_rows = """
{prompt}. 
here are the first rows of the input file, use them to understand the file structure: 
{header_rows}
"""


def get_code(prompt: str, model="gpt-3.5-turbo", header_rows=None):
    openai.api_key = os.getenv("OPENAI_API_KEY")

    if header_rows:
        prompt = prompt_with_rows.format(prompt=prompt, header_rows=header_rows)

    response = openai.ChatCompletion.create(
        model=model,
        messages=[
            {"role": "system", "content": f"{SYSTEM}"},
            {"role": "user", "content": f"{prompt}"},
        ],
        temperature=0.0,
        max_tokens=2048
        )

    code = response["choices"][0]["message"]["content"]
    code = code.strip("```")

    return code






from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='charmr',
    version='1.0.6',
    description='A Magical AI File Converter',
    url='https://github.com/harlev/charmr',
    author='Ron Harlev',
    author_email='harlev@gmail.com',
    packages=find_packages(),
    install_requires=[
        'openai',
        'python-dotenv'
    ],
    license='MIT',
    python_requires='>=3.8',
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={
        'console_scripts': [
            'charmr=charmr.main:main'
        ]
    }
)



import unicodedata
import re


def slugify(value, allow_unicode=False):
    """
    Taken from https://github.com/django/django/blob/master/django/utils/text.py
    Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
    dashes to single dashes. Remove characters that aren't alphanumerics,
    underscores, or hyphens. Convert to lowercase. Also strip leading and
    trailing whitespace, dashes, and underscores.
    """
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize('NFKC', value)
    else:
        value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
    value = re.sub(r'[^\w\s-]', '', value.lower())
    return re.sub(r'[-\s]+', '-', value).strip('-_')


def get_file_name(alias):
    return f"{slugify(alias)}.charmr"


def save_code(code_str, alias):
    file_name = get_file_name(alias)
    with open(file_name, 'w') as f:
        f.write(code_str)


def load_code(alias):
    file_name = get_file_name(alias)
    with open(file_name, 'r') as f:
        code_str = f.read()

    return code_str
